/**
 * 
 */
package ru.amse.ksenofontova.jina;

import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Date;
import java.util.List;

import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import ru.amse.ksenofontova.jina.io.LoadException;
import ru.amse.ksenofontova.jina.io.MainWindowParametersReader;
import ru.amse.ksenofontova.jina.io.MainWindowParametersWriter;
import ru.amse.ksenofontova.jina.io.SaveException;
import ru.amse.ksenofontova.jina.io.TasksReader;
import ru.amse.ksenofontova.jina.io.ViewParametersReader;
import ru.amse.ksenofontova.jina.io.ViewParametersWriter;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;
import ru.amse.ksenofontova.jina.model.impl.RepositoryManagerFactory;
import ru.amse.ksenofontova.jina.view.View;

/**
 * It's a main class, its <code>main</code> method runs the application.
 * This class is <code>JFrame</code> only function of which is create toolbar,
 * create <code>View</code> which is responsible for the hole application
 * image, add them to the contant and run the application. It also tries
 * to load data befor start and save parameters befor closing.
 * @author Eugenia
 * 
 */
@SuppressWarnings("serial")
public class MainWindow extends JFrame implements MainParameters {
    
    private JToolBar createToolBar(List<List<Action>> actions) {
        JToolBar toolBar = new JToolBar(); 
        toolBar.setFloatable(false);

        for (List<Action> list: actions) {
        	if(list.size() == 1){
        		toolBar.add(new JButton(list.get(0)));
        	} else {
        		ButtonGroup bg = new ButtonGroup();
        		for (Action action : list) {
					JToggleButton button = new JToggleButton(action);					
					toolBar.add(button);
					bg.add(button);
                    button.setSelected((Boolean)action.getValue("selected"));
				}        		
        	}
        	toolBar.addSeparator();
        }
        return toolBar;
    }

    /**
     * @param title
     */
    public MainWindow(String title) {
        super(title);
        addWindowListener(new ClosingListener());       
        
        
        // Creating application's own directory
        File dirFile = new File(UserHomePath);
        dirFile.mkdir();

        JPanel mainPanel = new JPanel();
        getContentPane().add(mainPanel); 
        
        View view;

        IRepositopyManager repositopyManager = null;
        // If there is no file to load data lets user work from scratch
        if (!new File(StorageFileName).exists()) {
            repositopyManager = RepositoryManagerFactory.createRepositoryManager();
            initStorageManager(repositopyManager);            
        } else {
            try {
                repositopyManager = RepositoryManagerFactory
                    .createRepositoryManager(new TasksReader(StorageFileName));
            } catch( LoadException e ) {
                // lets user work creating storage manager from scratch
                // without saving previous data.
                int res = JOptionPane
                    .showConfirmDialog(
                        this,
                        e.getMessage()
                            + "\n Do you want to begin from scratch (lose previous data)?",
                        LoadException.MESSAGE, JOptionPane.YES_NO_OPTION,
                        JOptionPane.ERROR_MESSAGE);
                if (res == JOptionPane.YES_OPTION) {
                    repositopyManager = RepositoryManagerFactory
                        .createRepositoryManager();
                } else {
                    System.exit(1);
                }
            }
        }
        try {
            new ViewParametersReader(ViewParametersFileName).read();
        } catch( LoadException e ) {
        }

        view = new View(this, repositopyManager);
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(createToolBar(view.getActions()), BorderLayout.NORTH);

        mainPanel.add(view, BorderLayout.CENTER);        
        
        try {
            new MainWindowParametersReader(MainWindowParametersFileName).read();
        } catch( LoadException e ) {
            MainWindowParameters.restoreDefaults();
        }
        
        setPreferredSize(MainWindowParameters.Size);
        // TODO make it work!
        setMinimumSize(MainWindowParameters.DEFAULT_SIZE);
        pack();
        setLocation(MainWindowParameters.Location);

    }
    
    /**
     * @param repositopyManager
     */
    private void initStorageManager(IRepositopyManager repositopyManager) {
        Date today  = DateUtil.getToday();
        Date date1 = DateUtil.addDays(today, -1);
        Date date2 = DateUtil.addDays(today, 2);
        
        ITask task1 = repositopyManager.createDayTask(date1, today, "Buy food, cook dinner");
        task1.setExecutionState(ExecutionState.DONE);
        repositopyManager.add(task1);
        
        ITask task2 = repositopyManager.createDayTask(today, today, "Celebrate my birthday!!!");
        task2.setPriority(Priority.IMPORTANT);
        repositopyManager.add(task2);
        
        ITask task3 = repositopyManager.createDayTask(today, date2, "Read John's book, return it.");
        repositopyManager.add(task3);
        
        ITask task4 = repositopyManager.createDayTask(today, today, "Study french");
        task4.setTimeLimitation(TimeLimitation.PERMANENT); 
        repositopyManager.add(task4);
        
    }

    private class ClosingListener extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            try {
                new ViewParametersWriter(ViewParametersFileName).write();
                
                MainWindowParameters.Location = getLocation();
                MainWindowParameters.Size = getSize();
                new MainWindowParametersWriter(MainWindowParametersFileName).write();
            } catch( SaveException e1 ) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
            System.exit(0);
        }        
    }
    
    
    private void run() {
        setVisible(true);
	}

    public static void main(String[] args) {
        new MainWindow(MainWindowTitle).run();
    }

}
