/**
 * Contains classes for reading and writing objects.
 */
package ru.amse.ksenofontova.jina.io;

import java.awt.Dimension;
import java.awt.Point;
import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ru.amse.ksenofontova.jina.MainWindowParameters;

/**
 * Class for reading <code>MainWindowParameters'</code> fields from specified XML-file 
 * @author Eugenia
 */
public class MainWindowParametersReader implements MainWindowRarametersIOConstants {
    private final String myFileName;
    
    /**
     * Creates <code>MainWindowParametersReader</code> for reading 
     * <code>MainWindowParametersReader'</code> fields from specified XML-file.
     * @param fileName
     *        name of the given XML-file to read <code>MainWindowParametersReader'</code>
     *        fields from
     * 
     */
    public MainWindowParametersReader (String fileName) {
        myFileName = fileName;
    }
    
    /**
     * Reads and sets <code>MainWindowParametersReader'</code> fields from XML-file 
     * specified in constructor
     * @throws LoadException if there are any problems in reading data 
     * (some other Exceptions in loading mechanism occurs)
     */
    public void read() throws LoadException {
        try {
          DocumentBuilderFactory docFactory = DocumentBuilderFactory
              .newInstance();
          docFactory.setIgnoringElementContentWhitespace(true);
          DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
          Document doc = docBuilder.parse(new File(myFileName));
          Element rootElement = doc.getDocumentElement();
          NodeList properties = rootElement.getChildNodes();
          for (int i = 0; i < properties.getLength(); i++) {
              Node propertyNode = properties.item(i);
              if (propertyNode instanceof Element) { 
                  Element propertyElement = (Element) propertyNode;                  
                  String tagName = propertyElement.getTagName();
                  
                  if (LOCATION_TAG_NAME.equals(tagName)) {
                      MainWindowParameters.Location = parseLocation(propertyElement);
                  } else if (SIZE_TAG_NAME.equals(tagName)) {
                      MainWindowParameters.Size = parseSize(propertyElement);
                  }
              }    
          }
       } catch( Throwable e ) {
              throw new LoadException(e.getMessage());
       }
    }

    /**
     * Parses <code>Dimension</code> object which is to be interpreted as size
     * from the given <code>Element</code> 
     * @param propertyElement the Element to parse <code>Dimension</code> from
     * @return parsed <code>Dimension</code> object which is to be interpreted
     * as size
     */
    private Dimension parseSize(Element propertyElement) {        
        int width = Integer.parseInt(propertyElement.getAttribute(WIDTH_TAG_NAME));
        int height = Integer.parseInt(propertyElement.getAttribute(HEIGHT_TAG_NAME));
        return new Dimension(width, height);
    }

    /**
     * Parses <code>Point</code> object which is to be interpreted as location
     * from the given <code>Element</code> 
     * @param propertyElement the Element to parse <code>Point</code> from
     * @return parsed <code>Point</code> object which is to be interpreted
     * as location
     */
    private Point parseLocation(Element propertyElement) {
        int xPosition = Integer.parseInt(propertyElement.getAttribute(X_POSITION_TAG_NAME));
        int yPosition = Integer.parseInt(propertyElement.getAttribute(Y_POSITION_TAG_NAME));
        return new Point(xPosition, yPosition);
    }              

}
