/**
 * Contains representations of tasks which user can plan.
 */
package ru.amse.ksenofontova.jina.model;

import java.util.Collection;
import java.util.Date;

import ru.amse.ksenofontova.jina.MainParameters;

/**
 * Manages manipulations with <code>ITask</code> instances such ascreating,
 * adding and removing them from some abstract repository. Also allows
 * to make extracts from repository by specified task parameters.
 * @author Eugenia
 *
 */
public interface IRepositopyManager extends MainParameters {
    
    /**
     * Removes specified <code>ITask</code> element from the repository.
     * @param task to be romoved
     */
    public void remove(ITask task);
    
    /**
     * Creates <code>ITask</code> element with 
     * <code>Identity.DAY_TASK</code> identity and specified parameters.
     * @param begin <code>Date</code> represents the beginning of the period
     * that task is planning for
     * @param end begin <code>Date</code> represents the completion of the period
     * that task is planning for
     * @param text task description
     */
    public ITask createDayTask(Date begin, Date end, String text); 
    
    /**
     * Adds <code>ITask</code> element to the repository.
     * @param task to be added.
     */
    public void add(ITask task);    
   
    /**
     * Returns Collection of <code>Task</code> where each element
     * fully or partly belongs to specified time interval or is a
     * permanent task that has been planned befor the Date specified
     * by the <code>end</code> parameter. Returnes an empty collection if there are no
     * required tasks. Returnes all the tasks if both parameters are null.
     * @param begin the beginning of the time interval. If it is null result
     * contains all the tasks that have been planned befor (not after) the 
     * Date specified by the <code>end</code> parameter.
     * @param end the ending of the time interval. If it is null result
     * contains all the tasks that have been planned from the Date specified
     * by the <code>begin</code> parameter.
     * @return Collection of <code>Task</code> where each element
     * fully or partly belongs to specified time interval or is a
     * permanent task that has been planned befor the Date specified
     * by the <code>end</code> parameter. 
     * Returnes an empty collection if there are no required tasks. 
     * Returnes all the tasks if both parameters are null.
     */
    public Collection<ITask> getInRange(Date begin, Date end); 

}
