/**
 * Contains private realisations of tasks represented by public 
 * interfaces in package ru.amse.ksenofontova.jina.model.
 */
package ru.amse.ksenofontova.jina.model.impl;

import ru.amse.ksenofontova.jina.io.LoadException;
import ru.amse.ksenofontova.jina.io.TasksReader;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;

/**
 * Factory for creating concrete repository managers of the abstract type
 * <code>IRepositopyManager</code>
 * 
 * @author Eugenia
 * 
 */
public class RepositoryManagerFactory {

    private RepositoryManagerFactory() {
    }

    /**
     * Creates IRepositopyManager and loads tasks into it by the specefied TasksReader.
     * 
     * @return IRepositopyManager with tasks loaded by the specefied TasksReader
     * @param  TasksReader to read tasks into IRepositopyManager's repository
     * @throws LoadException
     *         if it is unable to load data from given file (not enought memory).
     */
    public static IRepositopyManager createRepositoryManager(TasksReader tasksReader)
        throws LoadException {
        return new CollectionRepositoryManager(tasksReader);
    }
    

    /**
     * Creates empty IRepositopyManager. It is to be used at the beginnig of 
     * the work with application when there is no any file to load tasks yet. 
     * 
     * @return IRepositopyManager with an empty repository (no tasks).
     */
    public static IRepositopyManager createRepositoryManager() {
        return new CollectionRepositoryManager();
    }

}
