/**
 * Contains tests for interfaces and public classes of the hole progect.
 */
package ru.amse.ksenofontova.jina.tests;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Calendar;
import java.util.Date;

import org.junit.BeforeClass;
import org.junit.Test;

import ru.amse.ksenofontova.jina.model.DateUtil;

/**
 * @author Eugenia
 *
 */
public class DateUtilTest {
    
    private static int myYear;
    private static int myMonth;
    private static int myDay;
    private static int myHour;
    private static int myMinute;
    private static int myDayOfWeek;
    private static Date myDate;
    
    @BeforeClass
    public static void setUpBeforeClass() throws Exception { 
        myYear = 2007;
        myMonth = Calendar.MARCH;
        myDay = 8;
        myHour = 19;
        myMinute = 30;
        myDayOfWeek = Calendar.THURSDAY;
        myDate = DateUtil.newDate(myYear, myMonth, myDay);
        myDate = DateUtil.setTime(myDate, myHour, myMinute); 
    }
    
    //------------get() & set()----------------
    @Test()
    public void testGetYear(){
        assertTrue(myYear == DateUtil.getYear(myDate));        
    }
           
    @Test()
    public void testGetMonth(){
        assertTrue(myMonth == DateUtil.getMonth(myDate));       
    }
    
    @Test()
    public void testGetDay(){
        assertTrue(myDay == DateUtil.getDay(myDate));        
    }

    @Test()
    public void testSetGetHour(){
        assertTrue(myHour == DateUtil.getHour(myDate));        
    }
    
    @Test()
    public void testSetGetMinute(){
        assertTrue(myMinute == DateUtil.getMinute(myDate));        
    }
    
    @Test()
    public void testSetGetDayOfWeek(){
        assertTrue(myDayOfWeek == DateUtil.getDayOfWeek(myDate));        
    }
    
    //------------IsWeekend-------------------
    @Test()
    public void testIsWeekend1(){
        Date date = DateUtil.newDate(2007, 2, 17);
        assertTrue(DateUtil.isWeekend(date));        
    }
    
    @Test()
    public void testIsWeekend2(){
        Date date = DateUtil.newDate(2007, 2, 18);
        assertTrue(DateUtil.isWeekend(date));        
    }
    
    @Test()
    public void testIsWeekend3(){
        Date date = DateUtil.newDate(2007, 2, 19);
        assertFalse(DateUtil.isWeekend(date));        
    }
    
    //------------IsInRange-------------------
    @Test()
    public void testIsInRange1(){        
        assertTrue(DateUtil.isInRange(myDate, null, null));
        assertTrue(DateUtil.isInRange(myDate, myDate, null));
        assertTrue(DateUtil.isInRange(myDate, null, myDate));
        assertTrue(DateUtil.isInRange(myDate, myDate, myDate));
    }
        
        
    @Test()
    public void testIsInRange2(){
        Date date1 = DateUtil.newDate(myYear, myMonth, myDay);
        Date date2 = DateUtil.newDate(myYear + 1, myMonth, myDay);
        assertTrue(DateUtil.isInRange(date1, null, date2));
        assertFalse(DateUtil.isInRange(date1, date2, date2));
        assertFalse(DateUtil.isInRange(date1, date2, null));
        assertTrue(DateUtil.isInRange(date2, date1, null));
        assertFalse(DateUtil.isInRange(date2, date1, date1));
        assertFalse(DateUtil.isInRange(date2, null, date1));
        
    }
    
    //------------Add--------------------------
    @Test()
    public void testAddDay1(){
        Date date1 = DateUtil.newDate(myYear, myMonth, myDay);
        date1 = DateUtil.addDays(date1, 3);
        Date date2 = DateUtil.newDate(myYear, myMonth, myDay + 3);
        assertTrue(date1.equals(date2));  
    }
    
    @Test()
    public void testAddDay2(){
        Date date1 = DateUtil.newDate(myYear, myMonth, myDay);
        date1 = DateUtil.addDays(date1, DateUtil.getDaysInMonth(date1));
        Date date2 = DateUtil.newDate(myYear, myMonth + 1, myDay);
        assertTrue(date1.equals(date2));  
    }
    
    //------------getDiff----------------------
    @Test()
    public void testGetDaysDiff1(){
        int diff = 3;
        Date date2 = DateUtil.addDays(myDate, diff);
        assertTrue(DateUtil.getDaysDiff(date2, myDate) == diff);  
    }
    
    @Test()
    public void testGetDaysDiff2(){
        int diff = -3;
        Date date2 = DateUtil.addDays(myDate, diff);
        assertTrue(DateUtil.getDaysDiff(date2, myDate) == diff);  
    }
    
    @Test()
    public void testGetDaysDiff3(){
        assertTrue(DateUtil.getDaysDiff(myDate, myDate) == 0);  
    }
    
    @Test()
    public void testGetDaysDiff4(){
        Date date1 = DateUtil.newDate(2006, 7, 19);
        date1 = DateUtil.setTime(date1, 14, 20);
        int diff = 120;
        Date date2 = DateUtil.addDays(date1, diff);
        assertTrue(DateUtil.getDaysDiff(date2, date1) == diff); 
    }
    
    // 1-24.03.07 don't work, 25-31.03.07 work!
    @Test()
    public void testGetDaysDiff5(){
        Date date1 = DateUtil.newDate(2007, 2, 19);
        date1 = DateUtil.setTime(date1, 14, 20);
        int diff = 31;
        Date date2 = DateUtil.addDays(date1, diff);
        assertTrue(DateUtil.getDaysDiff(date2, date1) == diff);  
    }
    
    //------------Constructors-----------------
    //TODO
    
    //-----------Time & Date Independence--------
    @Test()
    public void testSetTimeDependence(){
        Date date = DateUtil.newDate(myYear, myMonth, myDay);
        date = DateUtil.setTime(date, myHour, myMinute);
        assertTrue(myYear == DateUtil.getYear(date));
        assertTrue(myMonth == DateUtil.getMonth(date));
        assertTrue(myDay == DateUtil.getDay(date));
    }
    
        
    //------------Days In Month----------------
    @Test()
    public void testGetDaysInMonth1(){  
        Date date1 = DateUtil.newDate(2004, Calendar.FEBRUARY, 1);
        Date date2 = DateUtil.newDate(2021, Calendar.FEBRUARY, 1);
        assertTrue(DateUtil.getDaysInMonth(date1) == 29);
        assertTrue(DateUtil.getDaysInMonth(date2) == 28);
    }
    
    
    @Test()
    public void testGetDaysInMonth2(){  
        Date date = DateUtil.newDate(2004, Calendar.JANUARY, 31);
        assertTrue(DateUtil.getDaysInMonth(date) == 31);
    }   
    
    
    //------------Exception--------------------
    @Test(expected=IllegalArgumentException.class)
    public void testNegativeYear(){
        int wrongYear = -100;
        DateUtil.newDate(wrongYear, myMonth, myDay);        
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testNegativeMonth(){
        int wrongMonth = -100;
        DateUtil.newDate(myYear, wrongMonth, myDay);        
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testExceededMonth(){
        int wrongMonth = 100;
        DateUtil.newDate(myYear, wrongMonth, myDay);
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testNegativeDay(){
        int wrongDay = -100;
        DateUtil.newDate(myYear, myMonth, wrongDay);        
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testExceededDay(){
        int wrongDay = 100;
        DateUtil.newDate(myYear, myMonth, wrongDay);
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testNegativeHour(){
        int wrongHour = -100;
        DateUtil.setTime(myDate, wrongHour, 0);
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testExceededHour(){
        int wrongHour = 100;
        DateUtil.setTime(myDate, wrongHour, 0);
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testNegativeMinute(){
        int wrongMinute = -100;
        DateUtil.setTime(myDate, myHour, wrongMinute);
    }
    
    @Test(expected=IllegalArgumentException.class)
    public void testExceededMinute(){
        int wrongMinute = 100;
        DateUtil.setTime(myDate, myHour, wrongMinute);
    }

}
