/**
 * 
 */
package ru.amse.ksenofontova.jina.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;

import org.junit.Test;

import ru.amse.ksenofontova.jina.MainWindowParameters;
import ru.amse.ksenofontova.jina.io.LoadException;
import ru.amse.ksenofontova.jina.io.MainWindowParametersReader;



/**
 * @author Eugenia
 *
 */
public class MainWindowParametersReaderTest extends MainWindowParametersIOTestsParameters {
    
    @Test(expected=LoadException.class)
    public void testReadFromNonExistingFileException() throws LoadException {
        String nonExistingFileName = "NonExistingFileName";
        MainWindowParametersReader reader = new MainWindowParametersReader(nonExistingFileName);
        reader.read();              
    }
    
    @Test
    public void testReadFromNonExistingFileMessage() throws IOException {
        String nonExistingFileName = TESTS_PATH + "NonExistingFileName";
        MainWindowParametersReader reader = new MainWindowParametersReader(nonExistingFileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            
            String canonicalFileName = new File(nonExistingFileName)
                .getCanonicalPath();
            String message = LoadException.MESSAGE + "\n" + canonicalFileName
                + " (    )";            
            assertEquals("Wrong message. \n" + message ,message, e.getMessage());                    
        }        
    }
    
    @Test(expected=LoadException.class)
    public void testReadFromWrongFileException() throws LoadException {
        String wrongFileName = TESTS_PATH + "WrongFile.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(wrongFileName);
        reader.read();              
    }
    
    @Test
    public void testReadFromWrongFileMessage() throws IOException {
        String wrongFileName = TESTS_PATH + "WrongFile.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(wrongFileName);
        try {
            reader.read();
        } catch( LoadException e ) {            
            String message = LoadException.MESSAGE 
                + "\nContent is not allowed in prolog.";            
            assertEquals("Wrong message. Expected: \n" + message ,message, e.getMessage());                                        
        }        
    }
    
    @Test
    public void testReadFromDefaultFile() {
        String fileName = TESTS_PATH + "MWP-default.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testDefaultSize() {
        String fileName = TESTS_PATH + "MWP-default.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(MainWindowParameters.Size, MainWindowParameters.DEFAULT_SIZE);
    }
    
    @Test
    public void testDefaultLocation() {
        String fileName = TESTS_PATH + "MWP-default.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(MainWindowParameters.Location, MainWindowParameters.DEFAULT_LOCATION);
    }
    
    @Test
    public void testSize() {
        String fileName = TESTS_PATH + "MWP-Size.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(MainWindowParameters.Size, TEST_SIZE);
    }
    
    @Test
    public void testLocation() {
        String fileName = TESTS_PATH + "MWP-Location.config";
        MainWindowParametersReader reader = new MainWindowParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(MainWindowParameters.Location, TEST_LOCATION);
    }


}
