/**
 * Contains tests for interfaces and public classes of the hole progect.
 */
package ru.amse.ksenofontova.jina.tests;


import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.Date;

import org.junit.BeforeClass;
import org.junit.Test;

import ru.amse.ksenofontova.jina.io.SaveException;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;
import ru.amse.ksenofontova.jina.model.impl.RepositoryManagerFactory;

/**
 * Tests IRepositopyManager
 * @author Eugenia
 *
 */
public class StorageManagerTest {
    
    private static IRepositopyManager myStorageManager;
    
    private static ITask task1;
    private static ITask task2;
    private static ITask task3;
    private static ITask task4;
    private static ITask task5;
    private static ITask task6;
    private static ITask task7;  
    
    private static Date date1 = DateUtil.newDate(2007, 2, 8);
    private static Date date2 = DateUtil.newDate(2007, 2, 9);
    private static Date date3 = DateUtil.newDate(2007, 2, 10);   
    

    /**
     * @throws java.lang.Exception
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        myStorageManager = RepositoryManagerFactory.createRepositoryManager();
        
        task1 = myStorageManager.createDayTask(date1, date1, "task1");
        myStorageManager.add(task1);
        task1.setExecutionState(ExecutionState.DONE);
        
        task2 = myStorageManager.createDayTask(date2, date2, "task2");
        myStorageManager.add(task2);
        task2.setPriority(Priority.IMPORTANT);
        task2.setExecutionState(ExecutionState.CANCELED);
        
        task3 = myStorageManager.createDayTask(date3, date3, "task3");
        myStorageManager.add(task3);
        
        task4 = myStorageManager.createDayTask(date1, date2, "task4");
        myStorageManager.add(task4);
        task4.setExecutionState(ExecutionState.DONE);
        task4.setPriority(Priority.IMPORTANT);
        
        task5 = myStorageManager.createDayTask(date2, date3, "task5");
        myStorageManager.add(task5);
        task5.setExecutionState(ExecutionState.CANCELED);
        
        task6 = myStorageManager.createDayTask(date1, date3, "task6");
        myStorageManager.add(task6);
        task6.setPriority(Priority.IMPORTANT);
        
        task7 = myStorageManager.createDayTask(date2, date2, "task7");
        myStorageManager.add(task7);
        task7.setTimeLimitation(TimeLimitation.PERMANENT);        
      
    }
    
    /**
     * Tests presence of added and absence of removed element.
     * @throws SaveException 
     *
     */
    @Test
    public void testAddGetInRangeRemove() throws SaveException{
        Collection<ITask> tasks
            = myStorageManager.getInRange(date1, date1);        
        assertTrue(tasks.contains(task1));
        
        myStorageManager.remove(task1);
        tasks = myStorageManager.getInRange(date1, date1);        
        assertFalse(tasks.contains(task1));
        task1 = myStorageManager.createDayTask(date1, date1, "task1");
        myStorageManager.add(task1);
        task1.setExecutionState(ExecutionState.DONE);
        
    }
    
    /**
     * Tests absence of task3[date3, date3] and task2[date2, date2] 
     * in [date1, date1]
     *
     */   
    @Test
    public void testGetInRange1(){
        Collection<ITask> tasks
            = myStorageManager.getInRange(date1, date1);

        assertFalse(tasks.contains(task3));
        assertFalse(tasks.contains(task2));
    }
    

    /**
     * Tests presence of task1[date1, date1], task2[date2, date2],  
     * task3[date3, date3], task4[date1, date2],
     * task5[date2, date3] and task6[date1, date3] in [date1, date3]
     *
     */   
    @Test
    public void testGetInRange2(){
        Collection<ITask> tasks
            = myStorageManager.getInRange(date1, date3);
        
        assertTrue(tasks.contains(task1));
        assertTrue(tasks.contains(task2));
        assertTrue(tasks.contains(task3));
        assertTrue(tasks.contains(task4));
        assertTrue(tasks.contains(task5));
        assertTrue(tasks.contains(task6));
    }
    

    /**
     * Tests presence of task2[date2, date2], task4[date1, date2],
     * task5[date2, date3] and task6[date1, date3]
     *  in [date2, date3]
     *
     */   
    @Test
    public void testGetInRange3(){
        Collection<ITask> tasks
            = myStorageManager.getInRange(date2, date2);        
        assertTrue(tasks.contains(task2));
        assertTrue(tasks.contains(task4));
        assertTrue(tasks.contains(task5));
        assertTrue(tasks.contains(task6));
    }

    /**
     * Tests absence of task7(PERMANENT: [date2, date2]) in [date1, date1]
     *
     */   
    @Test
    public void testGetInRangePermanentTask1(){
        Collection<ITask> tasks
            = myStorageManager.getInRange(date1, date1);
        
        assertFalse(tasks.contains(task7));
    }
    
    
    /**
     * Tests presence of task7(PERMANENT: [date2, date2]) in [date2, date2]
     *
     */   
    @Test
    public void testGetInRangePermanentTask2(){
        Collection<ITask> tasks
            = myStorageManager.getInRange(date2, date2);
        
        assertTrue(tasks.contains(task7));
    }


    /**
     * Tests presence of task7(PERMANENT: [date2, date2]) in [date3, date3]
     *
     */   
    @Test
    public void testGetInRangePermanentTask3(){
        Collection<ITask> tasks
            = myStorageManager.getInRange(date3, date3);
        
        assertTrue(tasks.contains(task7));
    }   


}
