/**
 * Contains tests for interfaces and public classes of the hole progect.
 */
package ru.amse.ksenofontova.jina.tests;

import static org.junit.Assert.assertTrue;

import java.util.Date;

import org.junit.BeforeClass;
import org.junit.Test;

import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Identity;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;
import ru.amse.ksenofontova.jina.model.impl.RepositoryManagerFactory;


/**
 * @author Eugenia
 *
 */
public class TaskTest {
    
    private static IRepositopyManager myStorageManager;
    
    private Date date1 = DateUtil.newDate(2007, 1, 23);
    private Date date2 = DateUtil.newDate(2007, 2, 8);
    private Date date3 = DateUtil.newDate(2007, 4, 9);
    private String myText = "Test";
    
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        myStorageManager = RepositoryManagerFactory.createRepositoryManager();
    }
    
    //----------------Get & Set---------------------
    @Test()
    public void testGetIdentity(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        assertTrue(task.getIdentity() == Identity.DAY_TASK);
    }
    
    @Test()
    public void testGetBeginDate(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        assertTrue(date1.equals(task.getBeginDate()));
    }
    
    @Test()
    public void testGetEndDate(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        assertTrue(date2.equals(task.getEndDate()));
    }
    
    @Test()
    public void testGetText(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        assertTrue(myText.equals(task.getText()));
    }
    
    @Test()
    public void testGetPriority(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        task.setPriority(Priority.IMPORTANT);
        assertTrue(Priority.IMPORTANT.equals(task.getPriority()));
    }
    
    @Test()
    public void testGetTimeLimitation(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        task.setTimeLimitation(TimeLimitation.PERMANENT);
        assertTrue(task.isPermanent());
    }
    
    @Test()
    public void testGetExecutionState(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        task.setExecutionState(ExecutionState.CANCELED);
        assertTrue(ExecutionState.CANCELED.equals(task.getExecutionState()));
    }
    
    @Test()
    public void testSetGetBeginDate(){
        ITask task = myStorageManager.createDayTask(date2, date3, myText);
        task.setDates(date1, date2);        
        assertTrue(date1.equals(task.getBeginDate()));
    }
    
    @Test()
    public void testSetGetEndDate(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        task.setDates(date1, date3);        
        assertTrue(date3.equals(task.getEndDate()));
    }
    
    //----------------toString----------------------
    @Test()
    public void testToString(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        assertTrue(task.toString() != null);
    }
    
    //----------------Equals & HashCode-------------
    @Test()
    public void testEquals(){
        ITask task1 = myStorageManager.createDayTask(date1, date2, myText);
        ITask task2 = myStorageManager.createDayTask(task1.getBeginDate(), task1.getEndDate()
            , task1.getText());
        assertTrue(task1.equalsTask(task2));
    }
    
    @Test()
    public void testHashCode(){
        ITask task1 = myStorageManager.createDayTask(date1, date2, myText);
        ITask task2 = myStorageManager.createDayTask(task1.getBeginDate(), task1.getEndDate()
            , task1.getText());
        assertTrue(task1.hashCode() == task2.hashCode());
    }
    
    
    //----------------Exceptions--------------------
    @Test(expected = IllegalArgumentException.class)
    public void testReverseTimeLimit1(){
        myStorageManager.createDayTask(date3, date2, myText);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void testReverseTimeLimit2(){
        ITask task = myStorageManager.createDayTask(date2, date3, myText);
        task.setDates(date2, date1);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void testReverseTimeLimit3(){
        ITask task = myStorageManager.createDayTask(date1, date2, myText);
        task.setDates(date3, date2);
    }

}
