/**
 * 
 */
package ru.amse.ksenofontova.jina.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import org.junit.Test;

import ru.amse.ksenofontova.jina.io.LoadException;
import ru.amse.ksenofontova.jina.io.SaveException;
import ru.amse.ksenofontova.jina.io.TasksReader;
import ru.amse.ksenofontova.jina.io.TasksWriter;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;
import ru.amse.ksenofontova.jina.model.impl.RepositoryManagerFactory;


/**
 * @author Eugenia
 *
 */
public class TasksReaderTest implements TestsParameters { 
    
//    @BeforeClass
//    public static void setUpBeforeClass() throws Exception {
//        System.setProperty("line.separator", "\n");
//    }

    @Test(expected=LoadException.class)
    public void testReadFromNonExistingFileException() throws LoadException {
        String nonExistingFileName = "NonExistingFileName";
        TasksReader reader = new TasksReader(nonExistingFileName);
        RepositoryManagerFactory.createRepositoryManager(reader);                
    }
    
    @Test
    public void testReadFromNonExistingFileMessage() throws IOException {
        String nonExistingFileName = TESTS_PATH + "NonExistingFileName";
        TasksReader reader = new TasksReader(nonExistingFileName);
        try {
            RepositoryManagerFactory.createRepositoryManager(reader);
        } catch( LoadException e ) {
            
            String canonicalFileName = new File(nonExistingFileName)
                .getCanonicalPath();
            String message = LoadException.MESSAGE + "\n" + canonicalFileName
                + " (    )";
            assertEquals("Wrong message. \n " + message, message, e.getMessage());                                        
        }        
    }
    
    @Test(expected=LoadException.class)
    public void testReadFromWrongFileException() throws LoadException {
        String wrongFileName = TESTS_PATH + "WrongFile.config";
        TasksReader reader = new TasksReader(wrongFileName);
        RepositoryManagerFactory.createRepositoryManager(reader);              
    }
    
    @Test
    public void testReadFromWrongFileMessage() throws IOException {
        String wrongFileName = TESTS_PATH + "WrongFile.config";
        TasksReader reader = new TasksReader(wrongFileName);
        try {
            RepositoryManagerFactory.createRepositoryManager(reader);
        } catch( LoadException e ) {
            String message = LoadException.MESSAGE + "\n" 
            + "Content is not allowed in prolog.";
            assertEquals("Wrong message. Expected: \n" + message ,message, e.getMessage());                                       
        }        
    }
    
    @Test
    public void testEmpty() {
        String fileName = TESTS_PATH + "T-empty.jcal";
        TasksReader reader = new TasksReader(fileName);
        try {
            IRepositopyManager rm = RepositoryManagerFactory
                .createRepositoryManager(reader);
            assertTrue(rm.getInRange(null, null).size() == 0);
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void test1DefaultTask() {
        String fileName = TESTS_PATH + "T-1DefaultTask.jcal";
        TasksReader reader = new TasksReader(fileName);
        try {
            IRepositopyManager rm = RepositoryManagerFactory
                .createRepositoryManager(reader);            
            assertTrue(rm.getInRange(null, null).size() == 1);
            ITask testTask = rm.getInRange(null, null).iterator().next();
            
            Date date = DateUtil.newDate(2007, 5, 8);
            ITask task = rm.createDayTask(date, date, "Input text");
            assertTrue(testTask.equalsTask(task));
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void test1Task1() {
        IRepositopyManager rm = RepositoryManagerFactory
        .createRepositoryManager(); 
        String fileName = TESTS_PATH + "T-1Task-1.jcal";
        TasksReader reader = new TasksReader(fileName);
        TasksWriter writer = new TasksWriter(fileName);
        
        Date date = DateUtil.newDate(2007, 5, 8);
        ITask task = rm.createDayTask(date, date, "Test");
        task.setPriority(Priority.IMPORTANT);
        task.setTimeLimitation(TimeLimitation.PERMANENT);
        rm.add(task);
        try {
            writer.write(rm);
            rm.remove(task);
            reader.read(rm);
                        
            assertTrue(rm.getInRange(null, null).size() == 1);
            
            ITask testTask = rm.getInRange(null, null).iterator().next(); 
            assertTrue(testTask.equalsTask(task));
            
        } catch( SaveException e1 ) {
            fail(e1.getMessage());
        } catch( LoadException e2 ) {
            fail(e2.getMessage());
        }
    }
    
    @Test
    public void test1Task2() {
        IRepositopyManager rm = RepositoryManagerFactory
        .createRepositoryManager(); 
        String fileName = TESTS_PATH + "T-1Task-2.jcal";
        TasksReader reader = new TasksReader(fileName);
        TasksWriter writer = new TasksWriter(fileName);
        
        Date date = DateUtil.newDate(2007, 5, 8);
        ITask task = rm.createDayTask(date, date, "Test");
        task.setPriority(Priority.IMPORTANT);
        task.setExecutionState(ExecutionState.DONE);
        rm.add(task);
        try {
            writer.write(rm);
            rm.remove(task);
            reader.read(rm);
                        
            assertTrue(rm.getInRange(null, null).size() == 1);
            
            ITask testTask = rm.getInRange(null, null).iterator().next(); 
            assertTrue(testTask.equalsTask(task));
            
        } catch( SaveException e1 ) {
            fail(e1.getMessage());
        } catch( LoadException e2 ) {
            fail(e2.getMessage());
        }
    }
    
    @Test
    public void test2Tasks() {
        IRepositopyManager rm = RepositoryManagerFactory
        .createRepositoryManager(); 
        String fileName = TESTS_PATH + "T-2Tasks.jcal";
        TasksReader reader = new TasksReader(fileName);
        TasksWriter writer = new TasksWriter(fileName);
        
        Date date = DateUtil.newDate(2007, 5, 8);
        ITask task1 = rm.createDayTask(date, date, "Test1");
        task1.setPriority(Priority.IMPORTANT);
        task1.setExecutionState(ExecutionState.DONE);
        rm.add(task1);
        
        ITask task2 = rm.createDayTask(date, date, "Test2");
        task2.setPriority(Priority.IMPORTANT);
        task2.setExecutionState(ExecutionState.CANCELED);
        rm.add(task2);
        
        try {
            writer.write(rm);
            rm.remove(task1);
            rm.remove(task2);
            reader.read(rm);
                        
            assertTrue(rm.getInRange(null, null).size() == 2);
            for (ITask testTask: rm.getInRange(null, null)) {
                assertTrue(testTask.equalsTask(task1) 
                    || testTask.equalsTask(task2));
            }
            
        } catch( SaveException e1 ) {
            fail(e1.getMessage());
        } catch( LoadException e2 ) {
            fail(e2.getMessage());
        }
    }
    
}
