/**
 * 
 */
package ru.amse.ksenofontova.jina.tests;


import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;

import org.junit.BeforeClass;
import org.junit.Test;

import ru.amse.ksenofontova.jina.io.LoadException;
import ru.amse.ksenofontova.jina.io.ViewParametersReader;
import ru.amse.ksenofontova.jina.view.ViewParameters;

/**
 * @author Eugenia
 *
 */
public class ViewParametersReaderTest extends ViewParametersIOTestsParameters {
    
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        setDefaultValues(); 
        //System.setProperty("line.separator", "\n");
    }

    @Test(expected=LoadException.class)
    public void testReadFromNonExistingFileException() throws LoadException {
        String nonExistingFileName = "NonExistingFileName";
        ViewParametersReader reader = new ViewParametersReader(nonExistingFileName);
        reader.read();              
    }
    
    @Test
    public void testReadFromNonExistingFileMessage() throws IOException {
        String nonExistingFileName = TESTS_PATH + "NonExistingFileName";
        ViewParametersReader reader = new ViewParametersReader(nonExistingFileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            
            String canonicalFileName = new File(nonExistingFileName)
                .getCanonicalPath();
            String message = LoadException.MESSAGE + "\n" + canonicalFileName
                + " (    )";            
            assertEquals("Wrong message. Expected: \n" + message ,message, e.getMessage());                                        
        }        
    }
    
    @Test(expected=LoadException.class)
    public void testReadFromWrongFileException() throws LoadException {
        String wrongFileName = TESTS_PATH + "WrongFile.config";
        ViewParametersReader reader = new ViewParametersReader(wrongFileName);
        reader.read();              
    }
    
    @Test
    public void testReadFromWrongFileMessage() throws IOException {
        String wrongFileName = TESTS_PATH + "WrongFile.config";
        ViewParametersReader reader = new ViewParametersReader(wrongFileName);
        try {
            reader.read();
        } catch( LoadException e ) {            
            String message = LoadException.MESSAGE 
                + "\nContent is not allowed in prolog.";            
            assertEquals("Wrong message. Expected: \n" + message ,message, e.getMessage());                                       
        }        
    }
    
    @Test
    public void testReadFromDefaultFile() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testFirstVisibledate() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.getFirstVisibleDate(), TEST_DATE);
    }
    
    @Test
    public void testDefaultBorderColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.BorderColor, ViewParameters.DEFAULT_BORDER_COLOR);
    }   
     
    @Test
    public void testDefaultGridColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.GridColor, ViewParameters.DEFAULT_GRID_COLOR);
    }
    
    
    @Test
    public void testDefaultTodayGridColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.TodayGridColor, ViewParameters.DEFAULT_TODAY_GRID_COLOR);
    }   
    
    
    @Test
    public void testDefaultViewFontColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.ViewFontColor, ViewParameters.DEFAULT_VIEW_FONT_COLOR);
    }
        
        
    @Test
    public void testDefaultWeekendFontColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.WeekendFontColor, ViewParameters.DEFAULT_WEEKEND_FONT_COLOR);
    }  
        
        
    @Test
    public void testDefaultViewBackgroundColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.ViewBackgroundColor, ViewParameters.DEFAULT_VIEW_BACKGROUND_COLOR);
    } 
    
    
    @Test
    public void testDefaultWorkDayTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.WorkDayTasksPanelBackgroundColor
            , ViewParameters.DEFAULT_WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR);
    }    
        
    @Test
    public void testDefaultWeekendTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.WeekendTasksPanelBackgroundColor
            , ViewParameters.DEFAULT_WEEKEND_TASKS_PANEL_BACKGROUND_COLOR);
    }   
        
        
    @Test
    public void testDefaultPastTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.PastTasksPanelBackgroundColor
            , ViewParameters.DEFAULT_PAST_TASKS_PANEL_BACKGROUND_COLOR);
    }   
        
     
    @Test
    public void testDefaultTaskBorderColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.TaskBorderColor
            , ViewParameters.DEFAULT_TASK_BORDER_COLOR);
    }
        
    @Test
    public void testDefaultSelectedTaskBackgroundColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.SelectedTaskBackgroundColor
            , ViewParameters.DEFAULT_SELECTED_TASK_BACKGROUND_COLOR);
    }    
        
    @Test
    public void testDefaultSelectedTaskFontColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.SelectedTaskFontColor
            , ViewParameters.DEFAULT_SELECTED_TASK_FONT_COLOR);
    }   
        
    @Test
    public void testDefaultTaskBackgroundColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.TaskBackgroundColor
            , ViewParameters.DEFAULT_TASK_BACKGROUND_COLOR);
    } 

    @Test
    public void testDefaultTaskFontColor() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.TaskFontColor
            , ViewParameters.DEFAULT_TASK_FONT_COLOR);
    }
        
        
    @Test
    public void testDefaultTaskFont() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.getTaskFont()
            , ViewParameters.DEFAULT_TASK_FONT);
    }
    
    @Test
    public void testDefaultScale() {
        String fileName = TESTS_PATH + "VP-default.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
        assertEquals(ViewParameters.getScale()
            , ViewParameters.DEFAULT_SCALE);
    }  
        
    
    @Test
    public void testBorderColor() {
        String fileName = TESTS_PATH + "VP-BorderColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.BorderColor, TEST_COLOR);           
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testGridColor() {
        String fileName = TESTS_PATH + "VP-GridColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.GridColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
   
    @Test
    public void testPastTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH 
            + "VP-PastTasksPanelBackgroundColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.PastTasksPanelBackgroundColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testSelectedTaskBackgroundColor() {
        String fileName = TESTS_PATH 
            + "VP-SelectedTaskBackgroundColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.SelectedTaskBackgroundColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testSelectedTaskFontColor() {
        String fileName = TESTS_PATH + "VP-SelectedTaskFontColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.SelectedTaskFontColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testTaskBackgroundColor() {
        String fileName = TESTS_PATH + "VP-TaskBackgroundColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.TaskBackgroundColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testTaskBorderColor() {
        String fileName = TESTS_PATH + "VP-TaskBorderColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.TaskBorderColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testTaskFontColor() {
        String fileName = TESTS_PATH + "VP-TaskFontColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.TaskFontColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testViewBackgroundColor() {
        String fileName = TESTS_PATH + "VP-ViewBackgroundColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.ViewBackgroundColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testTodayGridColor() {
        String fileName = TESTS_PATH + "VP-TodayGridColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.TodayGridColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testWeekendFontColor() {
        String fileName = TESTS_PATH + "VP-WeekendFontColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.WeekendFontColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testViewFontColor() {
        String fileName = TESTS_PATH + "VP-ViewFontColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.ViewFontColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testWeekendTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-WeekendTasksPanelBackgroundColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.WeekendTasksPanelBackgroundColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testWorkDayTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-WorkDayTasksPanelBackgroundColor.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();
            assertEquals(ViewParameters.WorkDayTasksPanelBackgroundColor, TEST_COLOR);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testTaskFont() {
        String fileName = TESTS_PATH + "VP-TaskFont.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            reader.read();            
            assertEquals(ViewParameters.getTaskFont(), TEST_FONT);            
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testReadTwoParams() {
        String fileName = TESTS_PATH + "VP-BorderColor-TaskFont.config";
        ViewParametersReader reader = new ViewParametersReader(fileName);
        try {
            ViewParameters.BorderColor = ViewParameters.DEFAULT_BORDER_COLOR;
            ViewParameters.setTaskFont(ViewParameters.DEFAULT_TASK_FONT);
            reader.read();
            assertEquals(ViewParameters.BorderColor, TEST_COLOR);
            assertEquals(ViewParameters.getTaskFont(), TEST_FONT); 
        } catch( LoadException e ) {
            fail(e.getMessage());
        }
    }
}
