/**
 * 
 */
package ru.amse.ksenofontova.jina.tests;

import static org.junit.Assert.*;

import org.junit.BeforeClass;
import org.junit.Test;

import ru.amse.ksenofontova.jina.io.SaveException;
import ru.amse.ksenofontova.jina.io.ViewParametersWriter;
import ru.amse.ksenofontova.jina.view.ViewParameters;

/**
 * @author Eugenia
 *
 */
public class ViewParametersWriterTest extends ViewParametersIOTestsParameters {

    /**
     * @throws java.lang.Exception
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        setDefaultValues();
        System.setProperty("line.separator", "\n");
    }
    
    @Test
    public void testWriteDefaultValues() {
        String fileName = TESTS_PATH + "VP-default.config";
        String testFileName = TESTS_PATH + "VP-default-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }        
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testBorderColor() {
        String fileName = TESTS_PATH + "VP-BorderColor.config";
        String testFileName = TESTS_PATH + "VP-BorderColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.BorderColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.BorderColor = ViewParameters.DEFAULT_BORDER_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testGridColor() {
        String fileName = TESTS_PATH + "VP-GridColor.config";
        String testFileName = TESTS_PATH + "VP-GridColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.GridColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.GridColor = ViewParameters.DEFAULT_GRID_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }    
    
    @Test
    public void testPastTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-PastTasksPanelBackgroundColor.config";
        String testFileName = TESTS_PATH + "VP-PastTasksPanelBackgroundColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.PastTasksPanelBackgroundColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.PastTasksPanelBackgroundColor 
            = ViewParameters.DEFAULT_PAST_TASKS_PANEL_BACKGROUND_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testSelectedTaskBackgroundColor() {
        String fileName = TESTS_PATH + "VP-SelectedTaskBackgroundColor.config";
        String testFileName = TESTS_PATH + "VP-SelectedTaskBackgroundColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.SelectedTaskBackgroundColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.SelectedTaskBackgroundColor 
            = ViewParameters.DEFAULT_SELECTED_TASK_BACKGROUND_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testSelectedTaskFontColor() {
        String fileName = TESTS_PATH + "VP-SelectedTaskFontColor.config";
        String testFileName = TESTS_PATH + "VP-SelectedTaskFontColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.SelectedTaskFontColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.SelectedTaskFontColor 
            = ViewParameters.DEFAULT_SELECTED_TASK_FONT_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testTaskBackgroundColor() {
        String fileName = TESTS_PATH + "VP-TaskBackgroundColor.config";
        String testFileName = TESTS_PATH + "VP-TaskBackgroundColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.TaskBackgroundColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.TaskBackgroundColor 
            = ViewParameters.DEFAULT_TASK_BACKGROUND_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testTaskBorderColor() {
        String fileName = TESTS_PATH + "VP-TaskBorderColor.config";
        String testFileName = TESTS_PATH + "VP-TaskBorderColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.TaskBorderColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.TaskBorderColor 
            = ViewParameters.DEFAULT_TASK_BORDER_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testTaskFontColor() {
        String fileName = TESTS_PATH + "VP-TaskFontColor.config";
        String testFileName = TESTS_PATH + "VP-TaskFontColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.TaskFontColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.TaskFontColor 
            = ViewParameters.DEFAULT_TASK_FONT_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testViewBackgroundColor() {
        String fileName = TESTS_PATH + "VP-ViewBackgroundColor.config";
        String testFileName = TESTS_PATH + "VP-ViewBackgroundColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.ViewBackgroundColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.ViewBackgroundColor 
            = ViewParameters.DEFAULT_VIEW_BACKGROUND_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testTodayGridColor() {
        String fileName = TESTS_PATH + "VP-TodayGridColor.config";
        String testFileName = TESTS_PATH + "VP-TodayGridColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.TodayGridColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.TodayGridColor 
            = ViewParameters.DEFAULT_TODAY_GRID_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testWeekendFontColor() {
        String fileName = TESTS_PATH + "VP-WeekendFontColor.config";
        String testFileName = TESTS_PATH + "VP-WeekendFontColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.WeekendFontColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.WeekendFontColor 
            = ViewParameters.DEFAULT_WEEKEND_FONT_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testViewFontColor() {
        String fileName = TESTS_PATH + "VP-ViewFontColor.config";
        String testFileName = TESTS_PATH + "VP-ViewFontColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.ViewFontColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.ViewFontColor 
            = ViewParameters.DEFAULT_VIEW_FONT_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testWeekendTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-WeekendTasksPanelBackgroundColor.config";
        String testFileName = TESTS_PATH + "VP-WeekendTasksPanelBackgroundColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.WeekendTasksPanelBackgroundColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.WeekendTasksPanelBackgroundColor 
            = ViewParameters.DEFAULT_WEEKEND_TASKS_PANEL_BACKGROUND_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testWorkDayTasksPanelBackgroundColor() {
        String fileName = TESTS_PATH + "VP-WorkDayTasksPanelBackgroundColor.config";
        String testFileName = TESTS_PATH + "VP-WorkDayTasksPanelBackgroundColor-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.WorkDayTasksPanelBackgroundColor = TEST_COLOR;
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.WorkDayTasksPanelBackgroundColor 
            = ViewParameters.DEFAULT_WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR;
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }
    
    @Test
    public void testWriteTwoParams() {
        String fileName = TESTS_PATH + "VP-BorderColor-TaskFont.config";
        String testFileName = TESTS_PATH + "VP-BorderColor-TaskFont-test.config";
        ViewParametersWriter writer = new ViewParametersWriter(testFileName);
        ViewParameters.BorderColor = TEST_COLOR;
        ViewParameters.setTaskFont(TEST_FONT);
        try {
            writer.write();
        } catch( SaveException e ) {
            fail(e.getMessage());
        }
        ViewParameters.BorderColor = ViewParameters.DEFAULT_BORDER_COLOR;
        ViewParameters.setTaskFont(ViewParameters.DEFAULT_TASK_FONT);
        FileComparator fc = new FileComparator(testFileName, fileName);
        assertTrue(fc.compare());
    }

}
