/**
 * 
 */
package ru.amse.ksenofontova.jina.view;

import java.awt.FontMetrics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

import ru.amse.ksenofontova.jina.MainParameters;

/**
 * 
 * @author Eugenia
 *
 */
@SuppressWarnings("serial")
/*package*/ class FramePanel extends JPanel implements MainParameters {

    
    /**
     * Return x coordinate of the left border of the day cell specified by 
     * the given index which is equal the difference between this shown 
     * date and firstVisibleDate.
     * @param index in [0, ViewParameters.getScale()] 
     * @return
     */
    protected int xCoordinate(int index) {
        return (int)(getWidth() - 2 * ViewParameters.DEFAULT_BORDER_WIDTH
            - ViewParameters.getTimeColumnWidth())             
            * index / ViewParameters.getScale() 
            + ViewParameters.getTimeColumnWidth() + ViewParameters.DEFAULT_BORDER_WIDTH;
    }    
    
    /**
     * Draws specified text in the specified rectangle
     * @param g2D
     * @param text
     * @param x - x coordinate of the top left corner of the specified rectangle
     * @param y - y coordinate of the top left corner of the specified rectangle
     * @param width of the top left corner of the specified rectangle
     * @param height of the top left corner of the specified rectangle
     */
    protected void drawText(Graphics2D g2D, String text, int x, int y
        , int width, int height) {
        drawText(g2D, text, x, y, width, height, LEFT_ALIGNMENT);
    }
    
    /**
     * Draws specified text in the specified rectangle
     * @param g2D
     * @param text
     * @param x - x coordinate of the top left corner of the specified rectangle
     * @param y - y coordinate of the top left corner of the specified rectangle
     * @param width of the top left corner of the specified rectangle
     * @param height of the top left corner of the specified rectangle
     * @param horizontalAligment
     */
    protected void drawText(Graphics2D g2D, String text, int x, int y
        , int width, int height, float horizontalAligment) {        
        FontMetrics fontMetrics = g2D.getFontMetrics();
        if((fontMetrics.stringWidth(text.substring(0, 0)) > width) 
            || (fontMetrics.getHeight() > height)) {
            return;
        }
        int textSumHeight = fontMetrics.getLeading();
        int beginIndex = 0;
        int endIndex = text.length();
        while((textSumHeight + fontMetrics.getAscent() < height) 
            && (beginIndex < endIndex)) {
            
//            int whiteSpaceIndex = endIndex;
            String substr = text.substring(beginIndex, endIndex);
            while(fontMetrics.stringWidth(substr) > width) {
//                if(Character.isSpaceChar(text.charAt(whiteSpaceIndex))) {
//                    whiteSpaceIndex = endIndex;
//                }
                endIndex--;
                substr = text.substring(beginIndex, endIndex);
            }
//            endIndex = whiteSpaceIndex;
//            substr = text.substring(beginIndex, endIndex);
            
            drawString(g2D, substr, x, y + textSumHeight + fontMetrics.getAscent()
                , width, horizontalAligment);
            textSumHeight += fontMetrics.getHeight();
            beginIndex = endIndex;
            endIndex = text.length();
        }
    }
    
    /**
     * Draw specified string from the given baseline position according to
     * given aligment. Does not check whether given string is less in width
     * then the given one or not. If answer is negative it will draw string
     * in wrong position. Use drawText() method!
     * @param g2D
     * @param str
     * @param x
     * @param y
     * @param width
     * @param aligment
     */
    private void drawString(Graphics2D g2D, String str, int x, int y
        ,int width, float horizontalAligment) {
        if(horizontalAligment == LEFT_ALIGNMENT) {
            g2D.drawString(str, x, y);
        } else if(horizontalAligment == CENTER_ALIGNMENT) {
            int margin = (int)
                (width - g2D.getFontMetrics().stringWidth(str)) / 2;
            g2D.drawString(str, x + margin, y);
        } else if(horizontalAligment == RIGHT_ALIGNMENT) {
            int margin = (int)
            (width - g2D.getFontMetrics().stringWidth(str));
            g2D.drawString(str, x + margin, y);
        }
    }

}
