/**
 * 
 */
package ru.amse.ksenofontova.jina.view;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Calendar;
import java.util.Date;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import com.toedter.calendar.JCalendar;

import ru.amse.ksenofontova.jina.MainParameters;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;


/**
 * Dialog that shows task parameters for user to edit
 * @author Eugenia
 * 
 */
@SuppressWarnings("serial")
public class TaskEditingDialog extends JDialog implements MainParameters, displayErrorLimitMessage { 
    
    private final int myWidth = 370;
    private final int myHeight = 580;

    // Text parameters    
    public final static String NewTaskTitle 
        = MainWindowTitle + " - New Task";
    public final static String EditTaskTitle 
        = MainWindowTitle + " - Edit Task";
    
    
    private static String myTextAreaText = "Input text";
    
    private static String ImportantCheckBoxText = "Important";
    
    private static String TimeLimitsComponentTitle = "Time-limit";
    private static String LimitedButtonText        = "Limited";
    private static String PermanentButtonText      = "Permanent";
    private static String EndDateLabelText         = "End date: ";
    private static String BeginDateLabelText       = "Begin date: ";
    
    private static String ExecutionStatesComponentTitle = "Execution state";
    private static String InProcessButtonText           = "In process";
    private static String CanceledButtonText            = "Canceled";
    private static String DoneButtonText                = "Done";    
       
    private static final String CANCELED_ICON_NAME = "canceled.gif";    
    private static final String DONE_ICON_NAME = "done.gif";    
    private static final String INPROCESS_ICON_NAME = "inprocess.gif"; 
    
    private static String HelpButtonText   = "Help";
    private static String OkButtonText     = "OK";
    private static String CancelButtonText = "Cancel";
    
    private static final int  myRowsCount = 5;
    private static final int  myCharsInRowCount = 100;
    
    
    // Task Attributes
    private /*final*/ ITask myTask;
    private TimeLimitation myTaskTimeLimitation;
    private Date myTaskBeginDate;
    private Date myTaskEndDate;    
    private ExecutionState myTaskExecutionState;    
    
    // Control Elements
    private final JTextArea myTextArea;
    private final JCheckBox myImportantCheckBox;    
    
    private JRadioButton myPermanentButton;
    private JRadioButton myLimitedButton;
    private JLabel myErrorLimitLabel;
    private static String ErrorLimitMessage = "End date must be later than" +
    		" begin date!";
    
    private JLabel myImageLabel;
    private JRadioButton myInProcessButton;
    private JRadioButton myDoneButton;
    private JRadioButton myCanceledButton;
    private JCalendar myBeginCalendar;
    private JCalendar myEndCalendar;
    private JLabel myBeginDateInfoLabel;
    private JLabel myEndDateInfoLabel;
    private JLabel myBeginDateLabel;
    private JLabel myEndDateLabel;
    
    private static TaskEditingDialog ourTaskEditingDialog;
    private OKButtonAction myOkButtonAction;
    
     
    /**
     * Creates a modal TaskEditingDialog for task editing with the 
     * specified title, owner <code>Frame</code> and specified task
     * for editing.
     * @param owner the <code>Frame</code> from which the dialog is displayed
     * @param title  the <code>String</code> to display in the dialog's
     *          title bar
     * @param task the <code>ITask</code> implementation object to edit
     */
    private TaskEditingDialog(JFrame owner) {        
        super(owner, true);        

        final JPanel mainPanel = new JPanel();  
        mainPanel.setBorder(BorderFactory.createEmptyBorder
            ( DistanceBetweenOrdinaryElements
            , DistanceBetweenOrdinaryElements
            , DistanceBetweenOrdinaryElements
            , DistanceBetweenOrdinaryElements));
        
        mainPanel.setLayout(new BorderLayout());
        
        final Box mainBox = Box.createVerticalBox();
        
        // --- myTextArea
        myTextArea = createTextArea();
        mainBox.add(new JScrollPane(myTextArea));
        mainBox.add(Box
            .createVerticalStrut(DistanceBetweenOrdinaryElements));
        
        // --- myImportantCheckBox
        myImportantCheckBox = new JCheckBox(ImportantCheckBoxText);
        mainBox.add(myImportantCheckBox);         
        mainBox.add(Box
            .createVerticalStrut(DistanceBetweenOrdinaryElements));
        
        // --- TimeLimitationPanel
        mainBox.add(createTimeLimitationPanel());
        mainBox.add(Box
            .createVerticalStrut(DistanceBetweenOrdinaryElements));

        // --- myExecutionStatePanel
        mainBox.add(createExecutionStatePanel());
        mainBox.add(Box
            .createVerticalStrut(DistanceBetweenOrdinaryElements));
        mainBox.add(Box.createVerticalGlue());

        mainPanel.add(BorderLayout.CENTER, mainBox);

        // --- Help, OK, Cancel
        final JComponent bottomBox = createButtonsComponent();
        mainPanel.add(BorderLayout.SOUTH, bottomBox);
        
        addWindowListener(new ClosingListener());        

        getContentPane().add(mainPanel); 
        setPreferredSize(new Dimension(myWidth, myHeight));
        pack(); 
    }
    
    protected JTextArea createTextArea() {
        JTextArea textArea = new JTextArea(myTextAreaText, myRowsCount,
            myCharsInRowCount);
        textArea.setLineWrap(true);
        textArea.setWrapStyleWord(true);
        setLayout(new BorderLayout());
        return textArea;
    }
    
    protected JPanel createTimeLimitationPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        int margin = DistanceBetweenSimilarElements;
        panel.setBorder(new CompoundBorder
            (new TitledBorder(TimeLimitsComponentTitle)
            , new EmptyBorder(margin, margin, margin, margin)));
        
        // -- Buttons
        final Box datesButtonsBox = Box.createVerticalBox();
        ButtonGroup bg = new ButtonGroup();

        // Permanent button
        final Box permanentBox = Box.createHorizontalBox();
        myPermanentButton = new JRadioButton
                    (new TimeLimitationAction(PermanentButtonText
                        ,TimeLimitation.PERMANENT));
        permanentBox.add(myPermanentButton);
        bg.add(myPermanentButton);
        permanentBox.add(Box.createHorizontalGlue());
        datesButtonsBox.add(permanentBox);
        
        // Limited button
        final Box limitedBox = Box.createHorizontalBox();
        myLimitedButton = new JRadioButton
                    (new TimeLimitationAction(LimitedButtonText
                        ,TimeLimitation.LIMITED));
        limitedBox.add(myLimitedButton);
        bg.add(myLimitedButton);
        limitedBox.add(Box.createHorizontalGlue());
        datesButtonsBox.add(limitedBox);
        
        panel.add(datesButtonsBox);
        panel.add(Box.createVerticalStrut
            (DistanceBetweenSimilarElements));
        
        final Box datesBox = Box.createVerticalBox();
        
        // -- Info
        final JPanel datesInfoPanel = new JPanel();
        datesInfoPanel.setLayout(new GridLayout(1,2
            , DistanceBetweenSimilarElements,0));
        
        myBeginDateLabel = new JLabel(BeginDateLabelText);
        myBeginDateInfoLabel = new JLabel();
        datesInfoPanel.add(myBeginDateLabel);
        datesInfoPanel.add(myBeginDateInfoLabel);
        
        myEndDateLabel = new JLabel(EndDateLabelText);
        myEndDateInfoLabel = new JLabel();
        datesInfoPanel.add(myEndDateLabel);
        datesInfoPanel.add(myEndDateInfoLabel);
        datesBox.add(datesInfoPanel);
        datesBox.add(Box.createVerticalStrut
            (DistanceBetweenSimilarElements));

        final JPanel calendarsPanel = new JPanel();
        calendarsPanel.setLayout(new GridLayout(1,2
            , DistanceBetweenSimilarElements,0));
        
        myBeginCalendar = new JCalendar(CurrentLocale);
        myBeginCalendar.setWeekOfYearVisible(false);
        myBeginCalendar.setMaxDayCharacters(2);
        myBeginCalendar.setBorder(BorderFactory.createLineBorder
            (ViewParameters.BorderColor));
        calendarsPanel.add(myBeginCalendar);       
                
        myEndCalendar = new JCalendar(CurrentLocale);
        myEndCalendar.setWeekOfYearVisible(false);
        myEndCalendar.setMaxDayCharacters(2);
        myEndCalendar.setBorder(BorderFactory.createLineBorder
            (ViewParameters.BorderColor));
        
        CalendarListener calendarListener = new CalendarListener();
        myBeginCalendar.addPropertyChangeListener("calendar", calendarListener);
        myEndCalendar.addPropertyChangeListener("calendar", calendarListener);
        
        calendarsPanel.add(myEndCalendar);
        
        datesBox.add(calendarsPanel);
        panel.add(datesBox);
        
        // -- ErrorLimitLabel
        panel.add(Box.createVerticalStrut
            (DistanceBetweenSimilarElements));
        final Box labelBox = Box.createHorizontalBox();
        myErrorLimitLabel = new JLabel(" ");        
        myErrorLimitLabel.setForeground(Color.RED);        
        labelBox.add(myErrorLimitLabel);
        labelBox.add(Box.createHorizontalGlue());
        panel.add(labelBox);        
                
        return panel;
    } 
    
    protected JPanel createExecutionStatePanel() {
        JPanel panel = new JPanel(new GridLayout(1,2
            , DistanceBetweenSimilarElements,0));        
        
        panel.setBorder(new CompoundBorder
            (new TitledBorder(ExecutionStatesComponentTitle), new EmptyBorder
                (DistanceBetweenSimilarElements
                , DistanceBetweenSimilarElements
                , DistanceBetweenSimilarElements
                , DistanceBetweenSimilarElements)));
        
        // image - state indicator
        myImageLabel = new JLabel();
        myImageLabel.setVerticalAlignment(JLabel.CENTER);
        myImageLabel.setHorizontalAlignment(JLabel.CENTER);
        panel.add(myImageLabel);        
        
        Box stateButtonsBox = Box.createVerticalBox();
        ButtonGroup bg = new ButtonGroup();
        myInProcessButton = new JRadioButton
                    (new ExecutionStateAction(InProcessButtonText
                        , ExecutionState.IN_PROCESS, INPROCESS_ICON_NAME));
        stateButtonsBox.add(myInProcessButton);
        bg.add(myInProcessButton);        
        myDoneButton = new JRadioButton
                (new ExecutionStateAction(DoneButtonText
                    , ExecutionState.DONE, DONE_ICON_NAME));
        stateButtonsBox.add(myDoneButton);
        bg.add(myDoneButton);        
        myCanceledButton = new JRadioButton
                (new ExecutionStateAction(CanceledButtonText
                    , ExecutionState.CANCELED, CANCELED_ICON_NAME));
        stateButtonsBox.add(myCanceledButton);
        bg.add(myCanceledButton);
        
        panel.add(stateButtonsBox);
        return panel;
    }    


    /**
     * Creates component with dialog buttons such as "Help", "OK", "Cancel"
     * to put them on the bottom.
     * @return component with buttons 
     */
    protected JComponent createButtonsComponent() {
        Box buttonBox = Box.createHorizontalBox();
        // Help
        JButton helpButton = new JButton(HelpButtonText);
        buttonBox.add(helpButton);
        buttonBox.add(Box.createHorizontalStrut
            (DistanceBetweenDifferentElements));
        buttonBox.add(Box.createHorizontalGlue());
        // OK
        myOkButtonAction = new OKButtonAction(OkButtonText);
        buttonBox.add(new JButton(myOkButtonAction));
        buttonBox.add(Box.createHorizontalStrut
            (DistanceBetweenSimilarElements));
        // Cancel
        JButton cancelButton = new JButton(CancelButtonText);        
        cancelButton.addActionListener(new CancelButtonListener());
        buttonBox.add(cancelButton);
        return buttonBox;
    }
    
    
    /**
     * Sets <code>beginDate</code> attribute to the editing task. 
     * @param beginDate 
     */
    private void setBeginDate(Date beginDate) {        
        myTaskBeginDate = beginDate;
        displayBeginDate();
        displayErrorLimitMessage();
        myOkButtonAction.setEnabled(myOkButtonAction.mustBeEnabled());
                
    }

    /**
     * Sets <code>endDate</code> attribute to the editing task. 
     * @param endDate 
     */
    private void setEndDate(Date endDate) {        
        myTaskEndDate = endDate;
        displayEndDate();
        displayErrorLimitMessage();
        myOkButtonAction.setEnabled(myOkButtonAction.mustBeEnabled());
        //if(isTimeLimitCorrect()) {
//        if(myOkButtonAction.mustBeEnabled()) {
//            myErrorLimitLabel.setText(" ");
//        } else {
//            myErrorLimitLabel.setText(ErrorLimitMessage);
//        }   
    }
    
    
   private boolean isTimeLimitCorrect() {
       final boolean correct = 
           DateUtil.getDaysDiff(myTaskEndDate, myTaskBeginDate) >= 0;
       return correct;
   }

       
    private void setExecutionState(ExecutionState executionState) {
        if(myTaskExecutionState != executionState) {
            myTaskExecutionState = executionState;
            if(executionState != ExecutionState.IN_PROCESS) {
                setTimeLimitation(TimeLimitation.LIMITED);
            }
            displayExecutionState();
        }        
    }    
    
    private void setTimeLimitation(TimeLimitation timeLimitation) {
        if(myTaskTimeLimitation != timeLimitation) {
            myTaskTimeLimitation = timeLimitation;
            if(timeLimitation == TimeLimitation.PERMANENT) {
                setExecutionState(ExecutionState.IN_PROCESS);
            }
            displayTimeLimitation();
        }        
    }           
    
    private void displayTimeLimitation() {
        if(myTaskTimeLimitation == TimeLimitation.PERMANENT) {
            myPermanentButton.doClick();
        } else {
            myLimitedButton.doClick();
        }       
    }
    
    private void displayExecutionState() {
        switch(myTaskExecutionState) {
        case IN_PROCESS: 
            myInProcessButton.doClick(); break;
        case DONE: 
            myDoneButton.doClick(); break;
        case CANCELED: 
            myCanceledButton.doClick(); break;        
        }
    }  
    
    private void displayAllAttributesInDialog(ITask task) {
        myTask = task; 

        myTaskTimeLimitation = myTask.getTimeLimitation();
        myTaskBeginDate = myTask.getBeginDate();
        myTaskEndDate = myTask.getEndDate();        
        myTaskExecutionState = myTask.getExecutionState();        
       
        myTextArea.setText
            ((myTask.getText() == "") ? myTextAreaText : myTask.getText());
        
        myImportantCheckBox.setSelected
            (myTask.getPriority() == Priority.IMPORTANT);
         
        displayErrorLimitMessage();
        displayBeginDate();
        displayEndDate();
        displayTimeLimitation();        
        displayExecutionState();        
    }

    
    private void displayErrorLimitMessage() {
        if(myOkButtonAction.mustBeEnabled()) {
            myErrorLimitLabel.setText(" "); 
        } else {
            myErrorLimitLabel.setText(ErrorLimitMessage); 
        }
    }

    /**
     * 
     */
    private void displayEndDate() {
        myEndCalendar.setDate(myTaskEndDate);
        myEndDateInfoLabel.setText(DateUtil.getDateString(myTaskEndDate));
    }

    /**
     * 
     */
    private void displayBeginDate() {
        myBeginCalendar.setDate(myTaskBeginDate);
        myBeginDateInfoLabel.setText(DateUtil.getDateString(myTaskBeginDate));
    }

    
    private ITask getTask() {
        return myTask;
    }
    
    public static ITask getResult() {
        return ourTaskEditingDialog.getTask();
    }
    

    public static void showTaskEditingDialog(JFrame owner, String title, ITask task) {        
        if(ourTaskEditingDialog == null) {
            ourTaskEditingDialog = new TaskEditingDialog(owner);
        }
        ourTaskEditingDialog.setTitle(title);
        ourTaskEditingDialog.displayAllAttributesInDialog(task);
        ourTaskEditingDialog.setLocation(owner.getX() + 
            (owner.getWidth() - ourTaskEditingDialog.getWidth()) / 2, owner.getY());
        ourTaskEditingDialog.setVisible(true);
    }       

    /**
     * Imitates dialog closing. Sets null value for task to return
     */
    private void doClose() {
        myTask = null;        
        setVisible(false);
    }
    
    private class TimeLimitationAction extends AbstractAction {
        private static final long serialVersionUID = 1L;
        private TimeLimitation timeLimitation;
        
        public TimeLimitationAction(String name
            , TimeLimitation timeLimitation) {
            super(name);
            this.timeLimitation = timeLimitation;
        }

        public void actionPerformed(ActionEvent e) {
            setTimeLimitation(timeLimitation);
            final boolean enabled = (timeLimitation != TimeLimitation.PERMANENT);
            
            myEndCalendar.setEnabled(enabled);
            myEndDateInfoLabel.setEnabled(enabled);
            myEndDateLabel.setEnabled(enabled);
            displayErrorLimitMessage();            
            myOkButtonAction.setEnabled(myOkButtonAction.mustBeEnabled());
        }
                
    }
    
    
    private class ExecutionStateAction extends AbstractAction {
        private static final long serialVersionUID = 1L;
        private ExecutionState myExecutionState;
        private ImageIcon myIcon;
        
        public ExecutionStateAction(String name
            , ExecutionState exState, String iconName) {
            super(name);
            myExecutionState = exState;
            URL url = TaskComponent.class.getResource(IMAGES_PATH + iconName);           
            myIcon = new ImageIcon(url);
        }
        
        public void actionPerformed(ActionEvent e) {
            myImageLabel.setIcon(myIcon);
            setExecutionState(myExecutionState);
        }        
    }   
        
    private class ClosingListener extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            TaskEditingDialog.this.doClose();
        }
        
    }    
    
    
    private class OKButtonAction extends AbstractAction {
    	
    	public OKButtonAction(String name){
    		super(name);
    	}

		public void actionPerformed(ActionEvent e) {
			myTask.setText(myTextArea.getText());
            if(myImportantCheckBox.isSelected()) {
                myTask.setPriority(Priority.IMPORTANT);
            } else {
                myTask.setPriority(Priority.NORMAL);
            }            
            myTask.setTimeLimitation(myTaskTimeLimitation);
            myTask.setDates(myTaskBeginDate, myTaskEndDate);
            myTask.setExecutionState(myTaskExecutionState);
            TaskEditingDialog.this.setVisible(false);
		}
		
		public boolean mustBeEnabled(){
			return !((myTaskTimeLimitation == TimeLimitation.LIMITED) && !isTimeLimitCorrect());
		}
    	
    }
    

    private class CancelButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            TaskEditingDialog.this.doClose();
        }
    }
    
    private class CalendarListener implements PropertyChangeListener{
        public void propertyChange(PropertyChangeEvent evt) {
            Calendar newCalendar = (Calendar)evt.getNewValue();
            if(evt.getSource() == myBeginCalendar) {
            	if(newCalendar.get(Calendar.DAY_OF_MONTH) != DateUtil.getDay(myTaskBeginDate)){
            		TaskEditingDialog.this.setBeginDate(newCalendar.getTime());
            	} 
            	if((newCalendar.get(Calendar.MONTH) == DateUtil.getMonth(myTaskBeginDate))
            			&& (newCalendar.get(Calendar.YEAR) == DateUtil.getYear(myTaskBeginDate))){
            		displayBeginDate();
            	}
            } else if(evt.getSource() == myEndCalendar) {
            	if(newCalendar.get(Calendar.DAY_OF_MONTH) != DateUtil.getDay(myTaskEndDate)){
            		TaskEditingDialog.this.setEndDate(newCalendar.getTime());
            	}
            	if((newCalendar.get(Calendar.MONTH) == DateUtil.getMonth(myTaskEndDate))
            			&& (newCalendar.get(Calendar.YEAR) == DateUtil.getYear(myTaskEndDate))){
            		displayEndDate();
            	}
           }
        }
    }

}
