/**
 * 
 */
package ru.amse.ksenofontova.jina.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import ru.amse.ksenofontova.jina.io.SaveException;
import ru.amse.ksenofontova.jina.io.TasksWriter;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.view.actions.DeleteTaskAction;

/**
 * @author Eugenia
 *
 */
@SuppressWarnings("serial")
public class TasksPanel extends FramePanel {
    
    private final View myAppView; 
    private final List<TaskComponent> myTaskComponents;    
    private ITask mySelectedTask; 

    /**
     * 
     */
    public TasksPanel(View view) {
        super();
        setBackground(ViewParameters.WorkDayTasksPanelBackgroundColor);        
        myAppView = view;
        myTaskComponents = new LinkedList<TaskComponent>();        
        addMouseListener(new MyMouseListener());
        getInputMap(WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke 
            (KeyEvent.VK_DELETE, 0, false), DeleteTaskAction.class);
        getActionMap().put(DeleteTaskAction.class, new DeleteTaskAction(this));        
        
        setToolTipText("");
        
    }
    
    public Dimension getPreferredSize() {
        return new Dimension
            (super.getPreferredSize().width
            , myTaskComponents.size() * (ViewParameters.getCellHeight()
                + ViewParameters.DEFAULT_BORDER_WIDTH) 
                + 2 * ViewParameters.DEFAULT_BORDER_WIDTH);
    }
    
    
    /**
     * @param g2D
     */
    protected void drawVerticalGrid(Graphics2D g2D) {
        int height = getHeight();
        Date today = DateUtil.getToday();
        boolean isLineAlreadyDrawn = false;
        for (int i = 0; i < ViewParameters.getScale(); i++) {
            Date date = DateUtil.addDays(ViewParameters.getFirstVisibleDate(),
                i);
            int x = xCoordinate(i);
            int x1 = xCoordinate(i + 1);
            if (DateUtil.isWeekend(date)) {
                g2D.setColor(ViewParameters.WeekendTasksPanelBackgroundColor);
                g2D.fillRect(x + ViewParameters.DEFAULT_BORDER_WIDTH, 0
                    , x1 - x - ViewParameters.DEFAULT_BORDER_WIDTH, height);
            }            
            if (DateUtil.getDaysDiff(date, today) == 0) {
                g2D.setColor(ViewParameters.TodayGridColor);
                g2D.drawLine(x, 0, x, height);
                g2D.drawLine(x1, 0, x1, height);
                isLineAlreadyDrawn = true;
            } else {
                if (!isLineAlreadyDrawn) {
                    g2D.setColor(ViewParameters.GridColor);
                    g2D.drawLine(x, 0, x, height);
                }
                isLineAlreadyDrawn = false;
            }
        }
    }
    
    
    protected void drawPast(Graphics2D g2D) {
        for (int i = ViewParameters.getScale(); i > 0; i--) {
            Date date = DateUtil.addDays(ViewParameters.getFirstVisibleDate(), i - 1);
            Date today = DateUtil.getToday();            
            if( DateUtil.getDaysDiff(date, today) < 0 ) {
                g2D.setColor(ViewParameters.PastTasksPanelBackgroundColor);
                g2D.fillRect(xCoordinate(0), 0
                    , xCoordinate(i) - xCoordinate(0), getHeight());
                break;
            }
        } 
    }
    
    

    /**
     * @param g2D
     */
    protected void drawTaskComponents(Graphics2D g2D) {
        Date firstVisibleDate = ViewParameters.getFirstVisibleDate();
        Date lastVisibleDate = DateUtil.addDays(firstVisibleDate,
            ViewParameters.getScale() - 1);

        Collection<ITask> tasks = myAppView.getStorageManager().getInRange(
            firstVisibleDate, lastVisibleDate);
        myTaskComponents.clear();
        int yCoord = ViewParameters.DEFAULT_BORDER_WIDTH;
        for (ITask task: tasks) {
            TaskComponent component = new TaskComponent(this, task, yCoord);
            myTaskComponents.add(component);
            component.paint(g2D);
            
            yCoord += ViewParameters.getCellHeight() 
                // Whitespace between TaskComponent's
                + ViewParameters.DEFAULT_BORDER_WIDTH;
        }        
    }
    
    
    private TaskComponent getTaskComponentAt(int x, int y) {
        TaskComponent selectedComponent = null;
        for (TaskComponent component: myTaskComponents) {
            if(component.containsPosition(x, y)) {
                selectedComponent = component;
                break;
            }
        }
        return selectedComponent;
    }
    
    ITask getSelectedTask() {
        return mySelectedTask;
    }
    
    /**
     * @throws HeadlessException
     */
    public void createTask(Date date) throws HeadlessException {        
        ITask defaultTask = myAppView.getStorageManager()
            .createDayTask(date, date, "");
        TaskEditingDialog.showTaskEditingDialog(myAppView
            .getMainWindow(), TaskEditingDialog.NewTaskTitle, defaultTask);
        ITask newTask = TaskEditingDialog.getResult();
        if(newTask != null) {
            try {
                IRepositopyManager repositopyManager= myAppView.getStorageManager();
                repositopyManager.add(newTask);
                new TasksWriter(StorageFileName).write(repositopyManager);
            } catch( SaveException e ) {
                JOptionPane.showMessageDialog(myAppView, e.getMessage()
                    , SaveException.MESSAGE, JOptionPane.ERROR_MESSAGE);
            }
            mySelectedTask = newTask;            
            repaint();            
            getToolkit().getSystemEventQueue().postEvent
               (new ComponentEvent(this, ComponentEvent.COMPONENT_RESIZED));
        }
    }
    

    /**
     * @param selectedComponent
     * @throws HeadlessException
     */
    private void editTask(ITask task) {
        TaskEditingDialog.showTaskEditingDialog(myAppView
            .getMainWindow(), TaskEditingDialog.EditTaskTitle, task);
        ITask newTask = TaskEditingDialog.getResult();
        if (newTask != null) {
            try {
                new TasksWriter(StorageFileName).write(myAppView.getStorageManager());
                repaint();
                getToolkit().getSystemEventQueue().postEvent
                   (new ComponentEvent(this, ComponentEvent.COMPONENT_RESIZED));
            } catch( SaveException e ) {
                JOptionPane.showMessageDialog(myAppView, e.getMessage()
                    , SaveException.MESSAGE, JOptionPane.ERROR_MESSAGE);
            }  
        }
    }
    
    
    public void deleteSelectedTask() {
        if(mySelectedTask != null) {
            try {
                IRepositopyManager repositopyManager= myAppView.getStorageManager();
                repositopyManager.remove(mySelectedTask);
                new TasksWriter(StorageFileName).write(repositopyManager);
            } catch( SaveException e ) {
                JOptionPane.showMessageDialog(myAppView, e.getMessage()
                    , SaveException.MESSAGE, JOptionPane.ERROR_MESSAGE);
            }
            mySelectedTask = null;
            repaint();
            getToolkit().getSystemEventQueue().postEvent
               (new ComponentEvent(this, ComponentEvent.COMPONENT_RESIZED));
        }
    }

    
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2D = (Graphics2D)g;               
        drawVerticalGrid(g2D);
        
        g2D.setColor(ViewParameters.ViewBackgroundColor);
        g2D.fillRect(0, 0, ViewParameters.getTimeColumnWidth()
            + ViewParameters.DEFAULT_BORDER_WIDTH, getHeight());
        
        drawTaskComponents(g2D);
        drawPast(g2D);
    }
        
    
    public String getToolTipText(MouseEvent e){
        TaskComponent selectedComponent = getTaskComponentAt
            (e.getX(), e.getY());
        return (selectedComponent == null) ? null 
            : selectedComponent.getToolTipText();
    }
    
    class MyMouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {            
            TaskComponent selectedComponent = getTaskComponentAt
                (e.getX(), e.getY());
            mySelectedTask = (selectedComponent == null)
                ? null : selectedComponent.getTask();            
            if (e.getClickCount() == 2) {  
                if(mySelectedTask != null) {
                    editTask(mySelectedTask);
                } else {
                    int cellWidth = xCoordinate(2) - xCoordinate(1);
                    int selectedDateIndex = (e.getX() - xCoordinate(0))/cellWidth;
                    Date selectedDate = DateUtil.addDays
                        (ViewParameters.getFirstVisibleDate()
                            , selectedDateIndex);
                    createTask(selectedDate);
                }
            } else if (e.getClickCount() == 1) {                
                repaint();
            }
        }        
    }    
                
}
