/**
 * 
 */
package ru.amse.ksenofontova.jina.view;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;

import ru.amse.ksenofontova.jina.MainParameters;
import ru.amse.ksenofontova.jina.MainWindow;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.view.actions.GoTodayAction;
import ru.amse.ksenofontova.jina.view.actions.NewTaskAction;
import ru.amse.ksenofontova.jina.view.actions.ThreeDaysAction;
import ru.amse.ksenofontova.jina.view.actions.WeekScaleAction;

/**
 * @author Eugenia
 *
 */
@SuppressWarnings("serial")
public class View extends JPanel implements MainParameters {
    
    private static final String DAY_FORWARD_ROLLOVER_ICON_NAME = "day forward - rollover.gif";
    private static final String DAY_FORWARD_ICON_NAME = "day forward.gif";
    private static final String DAY_BACK_ROLLOVER_ICON_NAME = "day back - rollover.gif";
    private static final String DAY_BACK_ICON_NAME = "day back.gif";
    private static final String VIEW_BACK_ROLLOVER_ICON_NAME = "week back - rollover.gif";
    private static final String VIEW_BACK_ICON_NAME = "week back.gif";
    private static final String VIEW_FORWARD_ROLLOVER_ICON_NAME = "week forward - rollover.gif";    
    private static final String VIEW_FORWARD_ICON_NAME = "week forward.gif";
    
    public static final int BACK = -1;
    public static final int FORWARD = 1;
    
    public static final KeyStroke DAY_BACK_KEY_STROKE = KeyStroke.getKeyStroke 
        (KeyEvent.VK_LEFT, KeyEvent.CTRL_MASK, false);
    public static final KeyStroke DAY_FORWARD_KEY_STROKE = KeyStroke.getKeyStroke 
        (KeyEvent.VK_RIGHT, KeyEvent.CTRL_MASK, false);
    public static final KeyStroke VIEW_BACK_KEY_STROKE = KeyStroke.getKeyStroke 
        (KeyEvent.VK_LEFT, KeyEvent.ALT_MASK, false);
    public static final KeyStroke VIEW_FORWARD_KEY_STROKE = KeyStroke.getKeyStroke 
        (KeyEvent.VK_RIGHT, KeyEvent.ALT_MASK, false);
    
    public static final String DayBackShortDescription     = "Day back";
    public static final String DayForwardShortDescription  = "Day forward";
    public static final String PreviousViewShortDescription = "Previous period";
    public static final String NextViewShortDescription = "Next period";
    
    private final JFrame myWindow;
    private final IRepositopyManager myStorage;
    private final List<List<Action> > myActions;
    
    
    public View(MainWindow window, IRepositopyManager repositopyManager){
        super();
        myWindow = window;
        myStorage = repositopyManager;        
        setLayout(new BorderLayout());
        TasksPanel dayTasksPanel = new TasksPanel(this);
        JScrollPane scrollPane = new JScrollPane(dayTasksPanel
            , JScrollPane.VERTICAL_SCROLLBAR_ALWAYS
            , JScrollPane.HORIZONTAL_SCROLLBAR_NEVER); 
        add(scrollPane, BorderLayout.CENTER);  
        
        //TODO calculate border
        JPanel datesPanel = createDatesPanel();
        datesPanel.setBorder(BorderFactory.createEmptyBorder
            (DistanceBetweenSimilarElements, 1, 1, 16/*scrollWidth*/));
        add(datesPanel, BorderLayout.NORTH);
        
        myActions = new LinkedList<List<Action>>();
        
        List<Action> actionsList = new LinkedList<Action>();
        actionsList.add(new NewTaskAction(dayTasksPanel, ImageUtil.createIcon("new.gif")));
        myActions.add(actionsList);
        
        actionsList = new LinkedList<Action>();
        ThreeDaysAction threeDaysAction = new ThreeDaysAction(this, ImageUtil.createIcon("three days.gif"));
        threeDaysAction.putValue("selected", ViewParameters.getScale() == 3);
        actionsList.add(threeDaysAction);
        
        WeekScaleAction weekScaleAction = new WeekScaleAction(this, ImageUtil.createIcon("week.gif"));
        weekScaleAction.putValue("selected", ViewParameters.getScale() == 7);
        actionsList.add(weekScaleAction);
        myActions.add(actionsList);
        
        actionsList = new LinkedList<Action>();        
        actionsList.add(new GoTodayAction(this, ImageUtil.createIcon("today.gif")));
        myActions.add(actionsList);
               
    }
    
    public JFrame getMainWindow() {
        return myWindow;
    }
    
    public List<List<Action> > getActions(){
        return myActions;
    }
    
    protected IRepositopyManager getStorageManager(){
        return myStorage;
    }
        
    private JPanel createDatesPanel(){
        JPanel datesPanel = new JPanel(new GridLayout(2,1));
                
        Box monthYearBox = Box.createHorizontalBox();
        monthYearBox.add(Box.createHorizontalStrut
            (ViewParameters.getTimeColumnWidth()));        
        
        JButton viewBackButton = createRollDatesButton(new RollViewAction
            (BACK, PreviousViewShortDescription), VIEW_BACK_KEY_STROKE
            , VIEW_BACK_ICON_NAME, VIEW_BACK_ROLLOVER_ICON_NAME);        
        monthYearBox.add(viewBackButton);
        
        monthYearBox.add(Box.createHorizontalStrut(DistanceBetweenSimilarElements));
        
        JButton viewForwardButton = createRollDatesButton(new RollViewAction
            (FORWARD, NextViewShortDescription), VIEW_FORWARD_KEY_STROKE
            , VIEW_FORWARD_ICON_NAME, VIEW_FORWARD_ROLLOVER_ICON_NAME);
        monthYearBox.add(viewForwardButton);
        
        JPanel monthYearPanel = new MonthYearPanel();
        monthYearBox.add(monthYearPanel); 
        
        JButton dayBackButton = createRollDatesButton(new RollDayAction
            (BACK, DayBackShortDescription), DAY_BACK_KEY_STROKE
            , DAY_BACK_ICON_NAME, DAY_BACK_ROLLOVER_ICON_NAME);        
        monthYearBox.add(dayBackButton);
        
        monthYearBox.add(Box.createHorizontalStrut(DistanceBetweenSimilarElements));
        
        JButton dayForwardButton = createRollDatesButton(new RollDayAction
            (FORWARD, DayForwardShortDescription), DAY_FORWARD_KEY_STROKE
            , DAY_FORWARD_ICON_NAME, DAY_FORWARD_ROLLOVER_ICON_NAME);
        monthYearBox.add(dayForwardButton);

        
        
        datesPanel.add(monthYearBox);
        datesPanel.add(new DatesPanel());
        return datesPanel;
    }
    
    private class RollDayAction extends AbstractAction {      
        
        private final int myDirection;
        
        public RollDayAction(int direction, String shortDescription) {
            super();
            myDirection = direction;
            putValue(SHORT_DESCRIPTION, shortDescription);
        }
        
        public void actionPerformed(ActionEvent e) {
            Date newFirstVisibleDate = DateUtil.addDays
                (ViewParameters.getFirstVisibleDate(), myDirection * 1);
            ViewParameters.setFirstVisibleDate(newFirstVisibleDate);
            View.this.repaint();
        }
    }
    

    private class RollViewAction extends AbstractAction {
       
        private int myDirection;

        public RollViewAction(int direction, String shortDescription) {
            super();
            myDirection = direction;
            putValue(SHORT_DESCRIPTION, shortDescription);            
        }

        public void actionPerformed(ActionEvent e) {
            Date newFirstVisibleDate = DateUtil.addDays(ViewParameters
                .getFirstVisibleDate(), myDirection * ViewParameters.getScale());
            ViewParameters.setFirstVisibleDate(newFirstVisibleDate);
            View.this.repaint();
        }
    }
    
    
    private JButton createRollDatesButton(Action action, KeyStroke keyStroke
        , String iconName, String rolloverIconName) {
        JButton button = new JButton(action);
        button.setBackground(ViewParameters.ViewBackgroundColor);
        button.setFocusable(false);
        button.setMargin(null);
        button.setBorderPainted(false);
        button.setBorder(null);
        button.setContentAreaFilled(false);
        button.setIcon(ImageUtil.createIcon(iconName));
        button.setRolloverIcon(ImageUtil.createIcon(rolloverIconName));
        
        //TODO find better actionMapKey
        String actionMapKey = action.getClass().toString() + keyStroke.getKeyCode();
        button.getInputMap(WHEN_IN_FOCUSED_WINDOW).put(keyStroke, actionMapKey);
        button.getActionMap().put(actionMapKey, action); 
        
        return button;
    }

}
