/**
 * 
 */
package ru.amse.ksenofontova.jina.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.util.Date;

import javax.swing.JPanel;

import ru.amse.ksenofontova.jina.model.DateUtil;

/**
 * @author Eugenia
 *
 */
public class ViewParameters {
    
    // --- Default Parameters --- //    
    public static final int DEFAULT_BORDER_WIDTH = 1;
    public static final int DEFAULT_SCALE = 7;
    public static final Color DEFAULT_BORDER_COLOR = new Color(131, 140, 156);
    public static final Color DEFAULT_GRID_COLOR = new Color(180, 180, 180); 
    public static final Color DEFAULT_TODAY_GRID_COLOR = new Color(40, 70, 179);
    
    public static final Color DEFAULT_VIEW_FONT_COLOR = Color.BLACK;
    public static final Color DEFAULT_WEEKEND_FONT_COLOR = new Color(160, 40, 45);
    public static final Color DEFAULT_VIEW_BACKGROUND_COLOR = new Color(238, 238, 238);
        
    public static final Color DEFAULT_WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR = Color.WHITE;
    public static final Color DEFAULT_WEEKEND_TASKS_PANEL_BACKGROUND_COLOR = new Color(254, 247, 247);
    public static final Color DEFAULT_PAST_TASKS_PANEL_BACKGROUND_COLOR = new Color(125, 125, 125, 50);
        
    public static final Color DEFAULT_TASK_BORDER_COLOR = DEFAULT_BORDER_COLOR;    
    
    public static final Color DEFAULT_SELECTED_TASK_BACKGROUND_COLOR = new Color(107, 144, 188);
    public static final Color DEFAULT_SELECTED_TASK_FONT_COLOR = Color.WHITE;
    
    public static final Color DEFAULT_TASK_BACKGROUND_COLOR = new Color(232, 242, 254);    
    public static final Color DEFAULT_TASK_FONT_COLOR = Color.DARK_GRAY;
    
    public static final Font DEFAULT_TASK_FONT 
        = new Font("Dialog", Font.PLAIN, 12);
    
    // ---View--- 
    public static Color BorderColor = DEFAULT_BORDER_COLOR;
    public static Color GridColor = DEFAULT_GRID_COLOR; 
    public static Color TodayGridColor = DEFAULT_TODAY_GRID_COLOR;  
    
    
    // Visible days count
    private static int myScale = DEFAULT_SCALE;
    
    private static Date myFirstVisibleDate = null;
    
    public static int getScale(){
        return myScale;
    }
    
    public static void setScale(int newScale){        
        int scalesDiff = newScale - myScale;             
        // refresh myFirstVisibleDate
        myFirstVisibleDate = DateUtil.addDays(getFirstVisibleDate()
        , - scalesDiff / 2);
        myScale = newScale;
    }  
    
    public static Date getFirstVisibleDate() {
        if (myFirstVisibleDate == null) {
            myFirstVisibleDate = DateUtil.addDays(DateUtil.getToday(),
                - myScale / 2);
        }
        return myFirstVisibleDate;
    }
    
    public static void setFirstVisibleDate(Date date) {
        myFirstVisibleDate = date;
    }
    
    
    // --Dates drawing---
    public static Color ViewFontColor = DEFAULT_VIEW_FONT_COLOR;
    public static Color WeekendFontColor = DEFAULT_WEEKEND_FONT_COLOR;
    public static Color ViewBackgroundColor = DEFAULT_VIEW_BACKGROUND_COLOR;
         
    // --Tasks drawing---  
    public static Color WorkDayTasksPanelBackgroundColor = DEFAULT_WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR;
    public static Color WeekendTasksPanelBackgroundColor = DEFAULT_WEEKEND_TASKS_PANEL_BACKGROUND_COLOR;
    public static Color PastTasksPanelBackgroundColor = DEFAULT_PAST_TASKS_PANEL_BACKGROUND_COLOR;
        
    public static Color TaskBorderColor = DEFAULT_TASK_BORDER_COLOR;    
    
    public static Color SelectedTaskBackgroundColor = DEFAULT_SELECTED_TASK_BACKGROUND_COLOR;
    public static Color SelectedTaskFontColor = DEFAULT_SELECTED_TASK_FONT_COLOR;
    
    public static Color TaskBackgroundColor = DEFAULT_TASK_BACKGROUND_COLOR;    
    public static Color TaskFontColor = DEFAULT_TASK_FONT_COLOR;
    
    private static Font myTaskFont = DEFAULT_TASK_FONT;
    
    public static Font getTaskFont() {
        return myTaskFont;
    }   
    
    public static void setTaskFont(Font font) {
        myTaskFont = font;
        myCellHeight = calulateCellHeight(font);
    }
    
    private static int myCellHeight = calulateCellHeight(myTaskFont);
    
    public static int getCellHeight() {
        return myCellHeight;
    }

    private static int calulateCellHeight(Font font) {
        //TODO replace "new JPanel()" - find another way
        FontMetrics fontMetrics = (new JPanel()).getFontMetrics(font);
        return (fontMetrics.getHeight() + DEFAULT_BORDER_WIDTH)* 2 
        + fontMetrics.getLeading();
    }  
    
    private static int myTimeColumnWidth = calculateTimeColumnWidth(myTaskFont);
    
    public static int getTimeColumnWidth() {
        return myTimeColumnWidth;
    }
    
    private static int calculateTimeColumnWidth(Font font){ 
        FontMetrics fontMetrics = (new JPanel()).getFontMetrics(font);
        Date date = DateUtil.getToday();
        date = DateUtil.setTime(date, 22, 22);
        
        return fontMetrics.stringWidth(DateUtil.getTimeString(date)) 
            + myCellHeight / 2;        
    }

}
